import { Chalk } from "chalk";
import * as saxes from "saxes";
import { Request, Response } from "express";
declare enum FSXAContentMode {
    PREVIEW = "preview",
    RELEASE = "release"
}
declare enum ImageMapAreaType {
    CIRCLE = "circle",
    RECT = "rect",
    POLY = "poly"
}
declare enum LogLevel {
    DEBUG = 0,
    INFO = 1,
    WARNING = 2,
    ERROR = 3,
    NONE = 4
}
declare class Logger {
    private _logLevel;
    private _name;
    private chalk;
    constructor(logLevel: LogLevel, name: string, chalk?: Chalk);
    get logLevel(): LogLevel;
    debug(...args: any[]): void;
    log(...args: any[]): void;
    info(...args: any[]): void;
    warn(...args: any[]): void;
    error(...args: any[]): void;
}
declare class XMLParser {
    logger: Logger;
    constructor(logger: Logger);
    sanitizeXml(xml: string): string;
    parse(xml: string): Promise<RichTextElement[]>;
    getCurrentElement: (result: RichTextElement, path: string[]) => any;
    createRichTextElement(tag: saxes.SaxesTag): RichTextElement;
}
type AvailableLocaleParams = {
    navigationServiceURL: string;
    projectId: string;
    contentMode: string | ("preview" | "release");
};
declare namespace ROUTES {
    enum FSXAApiErrors {
        UNKNOWN_CONTENT_MODE = "The content mode must be preview or release.",
        UNKNOWN_API_MODE = "The api mode must be remote or proxy.",
        UNKNOWN_ERROR = "An unknown error occured. Please check the logs for more information.",
        UNKNOWN_REMOTE = "The specified remote project was not found in the configuration. [remotes]",
        MISSING_BASE_URL = "You do need to specify a baseUrl in proxy mode.",
        MISSING_API_KEY = "No CaaS-ApiKey was passed via the configuration. [apiKey]",
        MISSING_CAAS_URL = "No CaaS-URL was passed via the configuration. [caas]",
        MISSING_NAVIGATION_SERVICE_URL = "No CaaS-URL was passed via the configuration. [navigationService]",
        MISSING_PROJECT_ID = "No projectId was passed via the configuration. [projectId]",
        MISSING_TENANT_ID = "No tenantId was passed via the configuration. [tenantId]",
        NOT_AUTHORIZED = "Your passed ApiKey has no access to the requested resource",
        NOT_FOUND = "Resource could not be found",
        MISSING_REMOTE_LOCALE = "The specified remote project did not include a locale",
        MISSING_REMOTE_ID = "The specified remote project did not include an id",
        INVALID_LOCALE = "The specified locale is not valid, locale needs to be a string of format \"xx_YY\""
    }
    enum FSXAProxyRoutes {
        FETCH_ELEMENT_ROUTE = "/elements",
        FETCH_NAVIGATION_ROUTE = "/navigation",
        FETCH_BY_FILTER_ROUTE = "/filter",
        FETCH_PROPERTIES_ROUTE = "/properties",
        STREAM_CHANGE_EVENTS_ROUTE = "/change-stream"
    }
    enum FSXAContentMode {
        PREVIEW = "preview",
        RELEASE = "release"
    }
    enum ImageMapAreaType {
        CIRCLE = "circle",
        RECT = "rect",
        POLY = "poly"
    }
    enum HttpStatus {
        CONTINUE = 100,
        SWITCHING_PROTOCOLS = 101,
        PROCESSING = 102,
        EARLYHINTS = 103,
        OK = 200,
        CREATED = 201,
        ACCEPTED = 202,
        NON_AUTHORITATIVE_INFORMATION = 203,
        NO_CONTENT = 204,
        RESET_CONTENT = 205,
        PARTIAL_CONTENT = 206,
        AMBIGUOUS = 300,
        MOVED_PERMANENTLY = 301,
        FOUND = 302,
        SEE_OTHER = 303,
        NOT_MODIFIED = 304,
        TEMPORARY_REDIRECT = 307,
        PERMANENT_REDIRECT = 308,
        BAD_REQUEST = 400,
        UNAUTHORIZED = 401,
        PAYMENT_REQUIRED = 402,
        FORBIDDEN = 403,
        NOT_FOUND = 404,
        METHOD_NOT_ALLOWED = 405,
        NOT_ACCEPTABLE = 406,
        PROXY_AUTHENTICATION_REQUIRED = 407,
        REQUEST_TIMEOUT = 408,
        CONFLICT = 409,
        GONE = 410,
        LENGTH_REQUIRED = 411,
        PRECONDITION_FAILED = 412,
        PAYLOAD_TOO_LARGE = 413,
        URI_TOO_LONG = 414,
        UNSUPPORTED_MEDIA_TYPE = 415,
        REQUESTED_RANGE_NOT_SATISFIABLE = 416,
        EXPECTATION_FAILED = 417,
        I_AM_A_TEAPOT = 418,
        MISDIRECTED = 421,
        UNPROCESSABLE_ENTITY = 422,
        FAILED_DEPENDENCY = 424,
        PRECONDITION_REQUIRED = 428,
        TOO_MANY_REQUESTS = 429,
        INTERNAL_SERVER_ERROR = 500,
        NOT_IMPLEMENTED = 501,
        BAD_GATEWAY = 502,
        SERVICE_UNAVAILABLE = 503,
        GATEWAY_TIMEOUT = 504,
        HTTP_VERSION_NOT_SUPPORTED = 505
    }
    enum LogLevel {
        DEBUG = 0,
        INFO = 1,
        WARNING = 2,
        ERROR = 3,
        NONE = 4
    }
    class Logger {
        private _logLevel;
        private _name;
        private chalk;
        constructor(logLevel: LogLevel, name: string, chalk?: Chalk);
        get logLevel(): LogLevel;
        debug(...args: any[]): void;
        log(...args: any[]): void;
        info(...args: any[]): void;
        warn(...args: any[]): void;
        error(...args: any[]): void;
    }
    class XMLParser {
        logger: Logger;
        constructor(logger: Logger);
        sanitizeXml(xml: string): string;
        parse(xml: string): Promise<RichTextElement[]>;
        getCurrentElement: (result: RichTextElement, path: string[]) => any;
        createRichTextElement(tag: saxes.SaxesTag): RichTextElement;
    }
    type AvailableLocaleParams = {
        navigationServiceURL: string;
        projectId: string;
        contentMode: string | ("preview" | "release");
    };
    /**
     * This method fetches and aggregate available locale from navigation service.
     * The {@link AvailableLocaleParams AvailableLocaleParams} object defines options to specify your request.
     * Example call:
     *
     * ```typescript
     const availableLocales = await getAvailableLocales({
     navigationServiceURL: 'https://your.navigation-service.url/navigation',
     projectId: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
     contentMode: 'preview',
     })
     * ```
     * @param navigationServiceURL the url of navigation service
     * @param projectId the project id to use
     * @param contentMode the content mode to use, either 'preview' or 'release'
     * @returns an array of available locales in ISO format. e.g. ['en_GB', 'de_DE']
     */
    const getAvailableLocales: ({ navigationServiceURL, projectId, contentMode }: AvailableLocaleParams) => Promise<string[]>;
    class HttpError extends Error {
        statusCode: number;
        constructor(message: string, statusCode: number);
    }
    type ParamValidationResult<T extends ParamValidationErrorReason> = ParamValidationSuccess | ParamValidationError<T>;
    type ParamValidationSuccess = {
        valid: true;
    };
    /**
     * All Existing Reasons to reject param Validation
     */
    type ParamValidationErrorReason = "MISSING_PARAM_OBJECT" | "MISSING_LOCALE" | "MISSING_ID" | "MISSING_FILTERS";
    type ParamValidationError<T extends ParamValidationErrorReason = ParamValidationErrorReason> = Readonly<{
        valid: false;
        reason: T;
        statusCode: 400;
    }>;
    /**
     * Creates the correct ValidationError for the given reason.
     * @param reason
     * @returns
     */
    const createValidationError: <T extends ParamValidationErrorReason>(reason: T) => Readonly<{
        valid: false;
        reason: T;
        statusCode: 400;
    }>;
    /**
     * Checks for required Parameters
     * @param params
     * @returns result object indicating validity, and a reason if not valid.
     */
    const validateParamsFetchElement: (params: Partial<FetchElementParams> | null | undefined) => ParamValidationResult<"MISSING_ID" | "MISSING_LOCALE" | "MISSING_PARAM_OBJECT">;
    /**
     * Checks for required Parameters
     * @param params
     * @returns result object indicating validity, and a reason if not valid.
     */
    const validateParamsFetchNavigation: (params: Partial<FetchNavigationParams> | null | undefined) => ParamValidationResult<"MISSING_LOCALE" | "MISSING_PARAM_OBJECT">;
    /**
     * Checks for required Parameters
     * @param params
     * @returns result object indicating validity, and a reason if not valid.
     */
    const validateParamsFetchByFilter: (params: Partial<FetchByFilterParams> | null | undefined) => ParamValidationResult<"MISSING_PARAM_OBJECT" | "MISSING_FILTERS">;
    /**
     * Checks for required Parameters
     * @param params
     * @returns result object indicating validity, and a reason if not valid.
     */
    const validateParamsFetchProjectProperties: (params: Partial<FetchProjectPropertiesParams> | null | undefined) => ParamValidationResult<"MISSING_LOCALE" | "MISSING_PARAM_OBJECT">;
    const FETCH_ELEMENT_ROUTE = "/elements/:id";
    const FETCH_NAVIGATION_ROUTE = "/navigation";
    const FETCH_BY_FILTER_ROUTE = "/filter";
    const HEALTH_ROUTE = "/health";
    const STREAM_CHANGE_EVENTS_ROUTE = "/change-stream";
    const getFetchElementRoute: (id: string) => string;
    interface FetchNavigationRouteBody {
        initialPath?: string;
        locale?: string;
        filterContext?: unknown;
    }
    interface FetchElementRouteBody {
        id: string;
        locale: string;
        additionalParams?: Record<string, unknown>;
        remote?: string;
        filterContext?: unknown;
        normalized?: boolean;
    }
    interface FetchByFilterBody {
        locale: string;
        filters: QueryBuilderQuery[];
        page?: number;
        pagesize?: number;
        sort?: SortParams[];
        additionalParams: Record<string, unknown>;
        remote: string;
        filterContext?: unknown;
        normalized?: boolean;
    }
    interface FetchProjectPropertiesBody {
        locale: string;
        additionalParams: Record<string, unknown>;
        resolver?: string[];
        filterContext?: unknown;
        normalized?: boolean;
    }
    type FetchWrapperResult<T = any> = FetchWrapperSuccess<T> | FetchWrapperError;
    type FetchWrapperSuccess<T = any> = {
        success: true;
        data: T;
    };
    type FetchWrapperError = {
        success: false;
        error: {
            statusCode: number;
            message: string;
        };
        originalError: Error | ParamValidationError;
    };
    /**
     * Useful Error Messages for Validation Errors.
     * Can be used as an Error Message to send to the client
     */
    const PARAM_VALIDATION_ERROR_TO_MESSAGE: Record<ParamValidationError["reason"], string>;
    /**
     * Provides Wrapper Functions fot the given api.
     * @param api FSXAApi to wrap
     * @param loggerName Creates a logger with the given name
     * @returns
     */
    const useEndpointIntegrationWrapper: (api: FSXARemoteApi, loggerName?: string) => {
        fetchElementWrapper: (params: FetchElementRouteBody, logContext?: string) => Promise<FetchWrapperResult>;
        fetchNavigationWrapper: (params: FetchNavigationRouteBody, logContext?: string) => Promise<FetchWrapperResult<NavigationData | null>>;
        fetchByFilterWrapper: (params: FetchByFilterBody, logContext?: string) => Promise<FetchWrapperResult<FetchResponse>>;
        fetchProjectPropertiesWrapper: (params: FetchProjectPropertiesBody, logContext?: string) => Promise<FetchWrapperResult<ProjectProperties | NormalizedProjectPropertyResponse | null>>;
    };
    type buildNavigationServiceURLParams = {
        locale?: string;
        initialPath?: string;
        all?: boolean;
    };
    type buildCaaSUrlParams = {
        filters?: QueryBuilderQuery[];
        locale?: string;
        page?: number;
        pagesize?: number;
        sort?: SortParams[];
        additionalParams?: Record<"keys" | string, any>;
        remoteProject?: string;
        id?: string;
    };
    /**
     * This class represents the `remote` variant of the FSXA API.
     */
    class FSXARemoteApi implements FSXAApi {
        mode: "remote";
        private _apikey;
        private _caasURL;
        private _navigationServiceURL;
        private _tenantID;
        private _projectID;
        private _remotes;
        private _contentMode;
        private _maxReferenceDepth?;
        private _customMapper?;
        private _queryBuilder;
        private _logger;
        private _navigationItemFilter?;
        private _caasItemFilter?;
        private _logLevel;
        private _enableEventStream;
        /**
         * The constructor of this class initializes the configuration for the api.
         *
         * @param config {@link FSXARemoteApiConfig FSXARemoteApiConfig}
         * @param config.apikey
         * @param config.caasURL
         * @param config.navigationServiceURL
         * @param config.tenantID
         * @param config.projectID
         * @param config.remotes optional {@link RemoteProjectConfiguration RemoteProjectConfiguration}
         * @param config.contentMode 'release' | 'preview'
         * @param config.maxReferenceDepth optional number to define the maximum depth of resolved objects
         * @param config.customMapper optional {@link CustomMapper CustomMapper}
         * @param config.filterOptions optional {@link RemoteApiFilterOptions RemoteApiFilterOptions} (EXPERIMENTAL)
         * @param config.logLevel the used {@link LogLevel LogLevel} for the API `(default LogLevel.ERROR)` - optional
         */
        constructor({ apikey, caasURL, navigationServiceURL, tenantID, projectID, remotes, contentMode, maxReferenceDepth, customMapper, filterOptions, logLevel }: FSXARemoteApiConfig);
        /**
         * Can be used in CaaS requests.
         * @returns an object with the configured apikey as value for the authorization key.
         */
        get authorizationHeader(): {
            authorization: string;
        };
        private verifyRemoteProjectExists;
        private getRemoteConfigById;
        /**
         * This methods builds an URL for the CaaS.
         * Based upon the optional {@link buildCaaSUrlParams buildCaaSUrlParams} object the returning url can link to any desired document.
         * @param id a specific CaaS document id
         * @param locale the locale of CaaS document (id of the document must be set)
         * @param remoteProject name of the remote project
         * @param additionalParams additional URL parameters
         * @param filters filters for CaaS documents - for more details read the [CaaS Platform documentation](https://docs.e-spirit.com/module/caas-platform/CaaS_Platform_Documentation_EN.html#use-of-filters)
         * @param page number of the page you want to access
         * @param pagesize number of the resulting CaaS documents
         * @returns a string that contains the CaaS URL with the configured parameters as query parameters
         */
        buildCaaSUrl({ id, locale, remoteProject: remoteProjectId, additionalParams, filters, page, pagesize, sort }?: buildCaaSUrlParams): string;
        /**
         * This fuction builds the URL for the NavigationService.
         * Based upon the optional {@link buildNavigationServiceURLParams buildNavigationServiceURLParams} object the returning url can link
         * to a seo route or a locale specific subtree of the navigation.
         * For more details how the Navigation Service works,
         * read the [Navigation Service documentation](https://navigationservice.e-spirit.cloud/docs/user/en/documentation.html).
         * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
         * @param initialPath can be provided when you want to access a subtree of the navigation
         * @returns {string} the Navigation Service url for either a subtree of or a complete navigation
         */
        buildNavigationServiceUrl({ locale, initialPath, all }?: buildNavigationServiceURLParams): string;
        /**
         * This method fetches the navigation from the configured navigation service.
         * The {@link FetchNavigationParams FetchNavigationParams} object defines options for the navigation.
         * Check {@link buildNavigationServiceUrl buildNavigationServiceUrl} to know which URL will be used.
         * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
         * @param initialPath optional value can be provided when you want to access a subtree of the navigation
         * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
         * @param filterContext an optional value with additional context used for filtering
         * @returns {Promise<NavigationData | null>} a Promise with the Navigation Service data or null
         */
        fetchNavigation({ locale, initialPath, fetchOptions, filterContext }: FetchNavigationParams): Promise<NavigationData | null>;
        private getFilteredNavigation;
        /**
         * This method fetches an element from the configured CaaS.
         * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
         * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
         * @typeParam T optional type parameter you can provide to get a typed CaaS object as result.
         * @param id the CaaS id of the element you want to fetch
         * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
         * @param additionalParams optional additional URL parameters
         * @param remoteProject optional name of the remote project
         * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
         * @returns {Promise<T>} a Promise with the mapped result
         */
        fetchElement<T = MappedCaasItem | any | null>({ id, locale, additionalParams, remoteProject, fetchOptions, filterContext, normalized }: FetchElementParams): Promise<any>;
        /**
         * This method fetches a filtered page from the configured CaaS.
         * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
         * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
         * Example call:
         *
         * ```typescript
         const englishMedia = await fetchByFilter({
         filters: [
         {
         field: 'fsType',
         value: 'Media',
         operator: ComparisonQueryOperatorEnum.EQUALS,
         },
         ],
         "en_GB",
         })
         * ```
         * @param filters array of {@link QueryBuilderQuery QueryBuilderQuery} to filter you request
         * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
         * @param page optional the number of the page you will get results from `(default = 1)` (must be greater than 0)
         * @param pagesize optional the number of document entries you will get back `(default = 30)` (must be greater than 0)
         * @param sort optional the parameter to sort the results by `(default = [])`.
         * @param additionalParams optional additional URL parameters
         * @param remoteProject optional name of the remote project
         * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
         * @returns the mapped and filtered response from the CaaS request,
         *    if `additionalParams.keys` are set, the result will be unmapped,
         *    if `data._embedded['rh:doc']` is undefined, the returning result will be the unmapped `data` object
         */
        fetchByFilter({ filters, locale, page, pagesize, additionalParams, remoteProject: remoteProjectId, fetchOptions, filterContext, sort, normalized }: FetchByFilterParams, mapper?: CaaSMapper): Promise<FetchResponse>;
        private filterMapResponse;
        // TODO: Fix unecessary array wrapping with a future major jump (as it's breaking)
        /**
         * This method fetches the project properties from the configured CaaS.
         * It uses {@link fetchByFilter fetchByFilter} to get them.
         * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
         * @param additionalParams optional additional URL parameters
         * @param resolve optional array of fsTypes that will be resolved `(default = 'GCAPage')`
         * @param filterContext
         * @param normalized
         * @returns the resolved project properties
         */
        fetchProjectProperties({ locale, additionalParams, resolve, filterContext, normalized }: {
            locale: string;
            additionalParams?: Record<string, any>;
            resolve?: string[];
            filterContext?: unknown;
            normalized?: boolean;
        }): Promise<ProjectProperties | NormalizedProjectPropertyResponse | null>;
        private fetchProjectPropertiesNormalized;
        /**
         * This method fetches a one-time secure token from the configured CaaS.
         * This token is used to establish the WebSocket connection.
         * @returns the secure token
         */
        fetchSecureToken(): Promise<string | null>;
        private buildStringifiedQueryParams;
        /**
         * @returns the configured apikey
         */
        get apikey(): string;
        /**
         * This method sets the string value of the apikey.
         * @param value the mandatory string value of the apikey
         */
        set apikey(value: string);
        /**
         * @returns the configured CaaS base url
         */
        get caasURL(): string;
        /**
         * This method sets the base url of the CaaS.
         * @param value the mandatory base url of the CaaS e.g. `https://customername-dev-caas-api.e-spirit.cloud`
         */
        set caasURL(value: string);
        /**
         * @returns the configured FirstSpirit project id
         */
        get projectID(): string;
        /**
         * This method sets the FirstSpirit project id
         * @param value the mandatory FirstSpirit project id
         */
        set projectID(value: string);
        /**
         * @returns the configured tenant id
         */
        get tenantID(): string;
        /**
         * This method sets the tenant id
         * @param value the mandatory tenant id (schema: `customername-stage`)
         */
        set tenantID(value: string);
        /**
         * @returns the configured NavigationService url
         */
        get navigationServiceURL(): string;
        /**
         * This method sets the url of the NavigationService.
         * @param value the mandatory url of the NavigationService e.g. `https://customername-stage-navigationservice.e-spirit.cloud/navigation`
         */
        set navigationServiceURL(value: string);
        /**
         * @returns the configured remote project configuration
         */
        get remotes(): RemoteProjectConfiguration;
        /**
         * This method sets the remote project configuration
         * @param value the mandatory remote project configuration
         * example:
         * ```typescript
         {
         "media": {
         "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
         "locale": "en_GB"
         }
         }
         ```
         */
        set remotes(value: RemoteProjectConfiguration);
        /**
         * @returns the configured content mode
         * `preview` links to the unreleased content
         * `release` links to the published content
         */
        get contentMode(): "preview" | "release";
        /**
         * This method sets the content mode
         * @param value the mandatory value of the content mode (allowed: `preview`, `release`)
         */
        set contentMode(value: "preview" | "release");
        /**
         * @returns the configured log level
         */
        get logLevel(): LogLevel;
        /**
         * Getter/Setter to enable the CaaS event stream
         * @returns true, if a event stream should pipe events from CaaS change events websocket
         */
        enableEventStream(enable?: boolean): boolean;
    }
    enum CaaSMapperErrors {
        UNKNOWN_BODY_CONTENT = "Unknown BodyContent could not be mapped.",
        UNKNOWN_FSTYPE = "Unknown fsType could not be mapped"
    }
    const DEFAULT_MAX_REFERENCE_DEPTH = 10;
    interface ReferencedItemsInfo {
        [identifier: string]: NestedPath[];
    }
    interface ResolvedReferencesInfo {
        [id: string]: MappedCaasItem | CaasApi_Item;
    }
    interface MapResponse {
        resolvedReferences: ResolvedReferencesInfo;
        mappedItems: (MappedCaasItem | CaasApi_Item)[];
        referenceMap: ReferencedItemsInfo;
    }
    class CaaSMapper {
        logger: Logger;
        api: FSXARemoteApi;
        locale: string | undefined;
        xmlParser: XMLParser;
        customMapper?: CustomMapper;
        referenceDepth: number;
        maxReferenceDepth: number;
        resolvedReferences: ResolvedReferencesInfo;
        // stores references to items of current Project
        _referencedItems: ReferencedItemsInfo;
        // stores References to remote Items
        _remoteReferences: {
            [projectId: string]: ReferencedItemsInfo;
        };
        // stores items that are being or have been processed and should not be fetched
        // Again. They cannot be added to resolvedReferences yet, since the fetch call
        // just started, when we need to keep track of them.
        _processedItems: Record<string, true>;
        constructor(api: FSXARemoteApi, locale: string | undefined, utils: {
            customMapper?: CustomMapper;
            referenceDepth?: number;
            maxReferenceDepth?: number;
        }, logger: Logger);
        addToResolvedReferences(item: MappedCaasItem | CaasApi_Item, remoteProjectId?: string): void;
        /**
         * unifies the two different id formats {id}.{locale} and {id} to {id}.{locale}
         * if remoteProjectConfiguration is passed, prefix with project id to avoid uuid clashes
         *
         * @param id uuid, may be of form {id}.{locale} or {id}
         * @param remoteProjectConfiguration remoteProjectConfig
         * @returns uuid of form {id}.{locale} or {remoteProjectId}#{id}.{locale}
         */
        unifyId(id: string, remoteProjectConfiguration?: RemoteProjectConfigurationEntry): string;
        /**
         * registers a referenced Item to be fetched later. If a remoteProjectId is passed,
         * the item will be fetched from the remote Project.
         * @param identifier item identifier
         * @param path after fetch, items are inserted at all registered paths
         * @param remoteProjectId optional. If passed, the item will be fetched from the specified project
         * @returns placeholder string
         */
        registerReferencedItem(identifier: string, path: NestedPath, remoteProjectId?: string, imageMapResolution?: string): string;
        buildPreviewId(identifier: string, remoteProjectLocale?: string): string;
        buildMediaUrl(url: string, rev?: number): string;
        mapDataEntry(entry: CaaSApi_DataEntry, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<DataEntry>;
        mapLinksInRichTextElements(richTextElements: RichTextElement[], path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<RichTextElement[]>;
        _mapPermissionGroup(group: CaaSAPI_PermissionGroup): PermissionGroup;
        mapDataEntries(entries: CaaSApi_DataEntries, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<DataEntries>;
        mapSection(section: CaaSApi_Section | CaaSApi_SectionReference, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Section>;
        mapContent2Section(content2Section: CaaSApi_Content2Section, remoteProjectLocale?: string): Promise<Section>;
        mapBodyContent(content: CaaSApi_Content2Section | CaaSApi_Section | CaaSApi_SectionReference, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<PageBodyContent>;
        mapPageBody(body: CaaSApi_Body, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<PageBody>;
        mapPageRef(pageRef: CaaSApi_PageRef, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Page>;
        mapProjectProperties(properties: CaaSApi_ProjectProperties, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ProjectProperties>;
        mapImageMapArea(area: CaaSApi_ImageMapArea, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ImageMapArea | null>;
        mapImageMap(imageMap: CaaSApi_CMSImageMap, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ImageMap>;
        mapGCAPage(gcaPage: CaaSApi_GCAPage, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<GCAPage>;
        mapDataset(dataset: CaaSApi_Dataset, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Dataset>;
        mapMediaPicture(item: CaaSApi_Media_Picture, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Image>;
        mapMediaPictureResolutionUrls(resolutions: CaaSApiMediaPictureResolutions, rev?: number): CaaSApiMediaPictureResolutions;
        mapMediaFile(item: CaaSApi_Media_File, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<File>;
        mapMedia(item: CaaSApi_Media, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Image | File | null>;
        // reassigned locale to avoid wrong preview ids
        // each item has its own locale
        private setLocaleFromCaasItem;
        mapFilterResponse(unmappedItems: (CaasApi_Item | any)[], additionalParams?: Record<string, any>, filterContext?: unknown, remoteProjectLocale?: string, remoteProjectId?: string): Promise<MapResponse>;
        /**
         * Calls ResolveReferences for currentProject and each RemoteProject
         *
         * @param data
         * @returns data
         */
        resolveAllReferences(filterContext?: unknown): Promise<void>;
        /**
         * This method will create a filter for all referenced items that are registered inside of the referencedItems
         * and fetch them in a single CaaS-Request. If remoteProjectId is set, referenced Items from the remoteProject are fetched
         * After a successful fetch all references in the json structure will be replaced with the fetched and mapped item
         */
        resolveReferencesPerProject(remoteProjectId?: string, filterContext?: unknown): Promise<void>;
        private getRemoteConfigForProject;
    }
    enum ComparisonQueryOperatorEnum {
        GREATER_THAN_EQUALS = "$gte",
        GREATER_THAN = "$gt",
        EQUALS = "$eq",
        IN = "$in",
        LESS_THAN = "$lt",
        LESS_THAN_EQUALS = "$lte",
        NOT_EQUALS = "$ne",
        NOT_IN = "$nin"
    }
    enum EvaluationQueryOperatorEnum {
        REGEX = "$regex"
    }
    enum LogicalQueryOperatorEnum {
        AND = "$and",
        NOT = "$not",
        NOR = "$nor",
        OR = "$or"
    }
    enum ArrayQueryOperatorEnum {
        ALL = "$all"
    }
    enum QueryBuilderErrors {
        MISSING_OPERATOR = "You have to specify an operator",
        UNKNOWN_OPERATOR = "Unknown operator passed",
        MISSING_FIELD = "The filter query requires a field",
        MISSING_VALUE = "The filter query requires a value",
        MISSING_FILTERS = "No filters property was specified",
        NOT_A_NUMBER = "This filter requires a number as value",
        NOT_AN_ARRAY = "This filter requires an array as value",
        NOT_A_STRING = "This filter requires a strinig as value",
        INVALID_REGEX = "This is not a valid regular expression"
    }
    class QueryBuilder {
        logger: Logger;
        constructor(logger: Logger);
        buildAll(filters: QueryBuilderQuery[]): MappedFilter[];
        build(filter: QueryBuilderQuery): MappedFilter | null;
    }
    /**
     * This class represents the functionality of the FSXA-API as a proxy.
     * To use this class, there must be an instance of FSXARemoteApi that is accessed here.
     * By using this class, no secrets are provided to the client.
     * @see FSXARemoteApi
     */
    class FSXAProxyApi implements FSXAApi {
        mode: "proxy";
        private _baseUrl;
        private _method;
        private _headers;
        private _logger;
        private _enableEventStream;
        private _filterContextProvider?;
        /**
         * This method requests the current state of baseUrl
         * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
         */
        get baseUrl(): string;
        /**
         * This method sets a new state of baseUrl
         * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
         */
        set baseUrl(value: string);
        /**
         * Creates a new instance with the connection to the FSXARemoteAPI
         * @param baseUrl specifies the URL to communicate with
         * @param logLevel specifies the restrictions of logs which will be displayed
         * @param filterOptions optional {@link ProxyApiFilterOptions ProxyApiFilterOptions} (EXPERIMENTAL)
         */
        constructor(baseUrl: string, logLevel?: LogLevel, filterOptions?: ProxyApiFilterOptions);
        /**
         * This methods fetches a specific element by its Id and its fetch options
         * @param id specifies the element that needs to be fetched
         * @param locale specifies the language e.g. de_DE
         * @param additionalParams sets parameters for the fetching process
         * @param remoteProject specifies the remote Project
         * @param fetchOptions specifies the options for the fetching process
         * @returns a JSON from the fetched element
         */
        fetchElement<T = Page | GCAPage | Dataset | Image | any | null>({ id, locale, additionalParams, remoteProject, fetchOptions }: FetchElementParams): Promise<T>;
        /**
         * This method fetches elements found by a specified Filter
         * @param filters specifies the filter that needs to be fetched
         * @param locale specifies the language
         * @param page specifies the page
         * @param pagesize specifies the number of pages
         * @param additionalParams specifies additional parameters for the fetching process
         * @param remoteProject specifies the remote project for the fetching process
         * @param fetchOptions specifies options in the fetching process
         * @param sort specifies the fields to sort. Default is by id descending. Multiple sortParams are possible. First parameter is prioritized over subsequent.
         * @returns a JSON from the fetched elements
         *
         * @example
         * ```
         * const response = await api.fetchByFilter({filters: [], locale: de'DE', page: 1, pagesize: 10, additionalParams: {}, remoteProject: 'my-project'})
         * ```
         */
        fetchByFilter({ filters, locale, page, pagesize, additionalParams, remoteProject, fetchOptions, sort }: FetchByFilterParams): Promise<FetchResponse>;
        /**
         * This method fetches the navigation
         * @param initialPath to identify the navigation
         * @param locale specifies the Language
         * @param fetchOptions specifies options in the fetching Process
         * @returns a JSON from the fetching process
         */
        fetchNavigation({ initialPath, locale, fetchOptions }: FetchNavigationParams): Promise<NavigationData | null>;
        /**
         * This method fetches Project Properties
         * @param locale specifies the language
         * @param resolver
         * @param fetchOptions specifies options in the fetching process
         * @returns a JSON from the fetching results
         */
        fetchProjectProperties({ locale, resolver, fetchOptions }: FetchProjectPropertiesParams): Promise<ProjectProperties | null>;
        /**
         * Getter/Setter to enable the CaaS event stream
         * @returns true, if a event stream should pipe events from CaaS change events websocket
         */
        enableEventStream(enable?: boolean): boolean;
        /**
         * This method initilises an `EventSoure` pointing to the CaaS change event stream.
         * @param additionalParams sets parameters for the fetching process
         * @param remoteProject specifies the remote Project
         * @returns an EventSource
         */
        connectEventStream({ remoteProject }?: ConnectEventStreamParams): EventSource;
        private fetch;
    }
    type FSXAApiSingletonInitOptions = {
        logLevel?: LogLevel;
        enableEventStream?: boolean;
    };
    class FSXAApiSingleton {
        private static _api;
        private static _logger;
        static init(api: FSXAApi, options?: FSXAApiSingletonInitOptions): FSXAApi;
        static get instance(): FSXAApi;
    }
    const eventStreamHandler: (api: FSXARemoteApi) => (req: Request, res: Response) => Promise<void>;
    interface MasterLocale {
        country: string;
        language: string;
        identifier: string;
    }
    interface CaaSApi_Lifespan {
        start: string;
        end?: string;
    }
    interface Lifespan {
        start: Date;
        end?: Date;
    }
    interface CaaSApi_Template {
        fsType: "PageTemplate" | "SectionTemplate" | "LinkTemplate";
        name: string;
        identifier: string;
        uid: string;
        uidType: string;
        displayName: string;
    }
    interface CaaSApi_Option {
        fsType: "Option";
        label: string;
        identifier: string;
    }
    interface CaaSApi_CMSInputCheckbox {
        fsType: "CMS_INPUT_CHECKBOX";
        name: string;
        value: CaaSApi_Option[];
    }
    interface CaaSApi_CMSInputRadioButton {
        fsType: "CMS_INPUT_RADIOBUTTON";
        name: string;
        value: CaaSApi_Option | null;
    }
    interface CaaSApi_CMSInputText {
        fsType: "CMS_INPUT_TEXT";
        name: string;
        value: string;
    }
    interface CaaSApi_CMSInputTextArea {
        fsType: "CMS_INPUT_TEXTAREA";
        name: string;
        value: string;
    }
    interface CaaSApi_CMSInputDOM {
        fsType: "CMS_INPUT_DOM";
        name: string;
        /**
         * This contains xml
         */
        value: string;
    }
    interface CaaSApi_CMSInputDOMTable {
        fsType: "CMS_INPUT_DOMTABLE";
        name: string;
        /**
         * This contains xml
         */
        value: string;
    }
    interface CaaSApi_CMSInputToggle {
        fsType: "CMS_INPUT_TOGGLE";
        name: string;
        value: boolean | null;
    }
    interface CaaSApi_CMSInputNumber {
        fsType: "CMS_INPUT_NUMBER";
        name: string;
        value: number;
    }
    interface CaaSApi_CMSInputLink {
        fsType: "CMS_INPUT_LINK";
        name: string;
        value: {
            template: CaaSApi_Template;
            formData: CaaSApi_DataEntries;
            metaFormData: CaaSApi_DataEntries;
        };
    }
    interface CaaSApi_CMSInputList {
        fsType: "CMS_INPUT_LIST";
        name: string;
        value: any[];
    }
    interface CaaSApi_CMSInputCombobox {
        fsType: "CMS_INPUT_COMBOBOX";
        name: string;
        value: CaaSApi_Option | null;
    }
    interface CaaSApi_CMSInputRadioButton {
        fsType: "CMS_INPUT_RADIOBUTTON";
        name: string;
        value: CaaSApi_Option | null;
    }
    interface CaaSApi_CMSInputDate {
        fsType: "CMS_INPUT_DATE";
        name: string;
        value: string | null;
    }
    interface Point2D {
        x: number;
        y: number;
    }
    interface CaaSApi_ImageMapArea {
        fsType: "Area";
        areaType: ImageMapAreaType;
        link: {
            template: CaaSApi_Template;
            formData: CaaSApi_DataEntries;
        } | null;
    }
    interface CaaSApi_ImageMapAreaCircle extends CaaSApi_ImageMapArea {
        areaType: ImageMapAreaType.CIRCLE;
        radius: number;
        center: Point2D;
    }
    interface CaaSApi_ImageMapAreaRect extends CaaSApi_ImageMapArea {
        areaType: ImageMapAreaType.RECT;
        leftTop: Point2D;
        rightBottom: Point2D;
    }
    interface CaaSApi_ImageMapAreaPoly extends CaaSApi_ImageMapArea {
        areaType: ImageMapAreaType.POLY;
        points: Point2D[];
    }
    interface CaaSApi_ImageMapMedia extends Pick<CaaSApi_Media, "fsType" | "name" | "displayName" | "identifier" | "uid" | "uidType" | "mediaType"> {
        url: string;
        pictureMetaData: Omit<CaaSApi_Media_Picture_Resolution_MetaData, "url">;
    }
    interface CaaSApi_CMSImageMap {
        fsType: "CMS_INPUT_IMAGEMAP";
        name: string;
        value: {
            fsType: "MappingMedium";
            media: CaaSApi_MediaRef;
            areas: CaaSApi_ImageMapArea[];
            resolution: {
                fsType: "Resolution";
                uid: string;
                width: number;
                height: number;
            };
        };
    }
    interface CaaSApi_FSDataset {
        fsType: "FS_DATASET";
        name: string;
        value: {
            fsType: "DatasetReference";
            target: {
                fsType: "Dataset";
                schema: string;
                entityType: string;
                identifier: string;
            };
        } | CaaSApi_DataEntry[] | null;
    }
    interface CaaSApi_FSButton {
        fsType: "FS_BUTTON";
        name: string;
        value: {};
    }
    interface CaaSApi_Card {
        fsType: "Card";
        identifier: string;
        uuid: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
    }
    interface CaaSApi_FSCatalog {
        fsType: "FS_CATALOG";
        name: string;
        value: CaaSApi_Card[] | null;
    }
    interface CaaSApi_Record {
        fsType: "Record";
        identifier: string;
        value: Record<string, any> | null;
    }
    interface CaaSApi_FSIndex {
        fsType: "FS_INDEX";
        name: string;
        dapType: string;
        value: CaaSApi_Record[];
    }
    interface CaaSApi_SectionBaseInfo {
        name: string;
        displayName: string;
        identifier: string;
    }
    interface CaaSApi_BaseRef {
        fsType: string;
        name: string;
        identifier: string;
        uid: string;
        uidType: string;
        url: string;
        remoteProject?: string;
        section?: CaaSApi_SectionBaseInfo;
    }
    interface CaaSApi_MediaRef extends CaaSApi_BaseRef {
        fsType: "Media";
        uidType: "MEDIASTORE_LEAF";
        mediaType: "PICTURE" | "FILE";
    }
    interface CaaSApi_GCARef extends CaaSApi_BaseRef {
        fsType: "GCAPage";
        uidType: "GLOBALSTORE";
        url: "";
    }
    interface CaaSApi_PageRefRef extends CaaSApi_BaseRef {
        fsType: "PageRef";
        uidType: "SITESTORE_LEAF";
    }
    interface CaaSApi_FSReference {
        fsType: "FS_REFERENCE";
        name: string;
        value: CaaSApi_BaseRef | CaaSApi_PageRefRef | CaaSApi_GCARef | CaaSApi_MediaRef | null;
    }
    interface CaaSAPI_PermissionGroup {
        groupName: string;
        groupPath: string;
    }
    interface CaaSAPI_PermissionActivity {
        activity: string;
        allowed: CaaSAPI_PermissionGroup[];
        forbidden: CaaSAPI_PermissionGroup[];
    }
    interface CaaSApi_CMSInputPermission {
        fsType: "CMS_INPUT_PERMISSION";
        name: string;
        value: CaaSAPI_PermissionActivity[];
    }
    interface PermissionGroup extends CaaSAPI_PermissionGroup {
        groupId: string;
    }
    interface PermissionActivity extends CaaSAPI_PermissionActivity {
        allowed: PermissionGroup[];
        forbidden: PermissionGroup[];
    }
    interface Permission extends CaaSApi_CMSInputPermission {
        type: string;
        value: PermissionActivity[];
    }
    interface DatasetRoute {
        pageRef: string;
        route: string;
    }
    type CaaSApi_DataEntry = CaaSApi_CMSInputCheckbox | CaaSApi_CMSInputCombobox | CaaSApi_CMSInputDOM | CaaSApi_CMSInputDOMTable | CaaSApi_CMSInputLink | CaaSApi_CMSInputList | CaaSApi_CMSInputNumber | CaaSApi_CMSInputText | CaaSApi_CMSInputTextArea | CaaSApi_CMSInputToggle | CaaSApi_CMSImageMap | CaaSApi_FSButton | CaaSApi_FSCatalog | CaaSApi_FSDataset | CaaSApi_FSIndex | CaaSApi_FSReference | CaaSApi_CMSInputRadioButton | CaaSApi_CMSInputDate | CaaSApi_Option | CaaSApi_CMSInputPermission;
    interface CaaSApi_DataEntries {
        [key: string]: CaaSApi_DataEntry;
    }
    interface CaaSApi_Content2Section {
        fsType: "Content2Section";
        identifier: string;
        schema: string;
        entityType: string;
        name: string;
        displayName: string;
        template: CaaSApi_Template;
        query: string | null;
        recordCountPerPage: number;
        maxPageCount: number;
        filterParams: {
            [key: string]: any;
        };
        ordering: {
            attribute: string;
            ascending: boolean;
        }[];
    }
    interface CaaSApi_Dataset {
        _id: string;
        fsType: "Dataset";
        schema: string;
        entityType: string;
        displayName: string;
        identifier: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        route: string;
        routes: DatasetRoute[];
        locale: {
            language: string;
            country: string;
        };
    }
    interface CaaSApi_Section {
        fsType: "Section" | "GCASection";
        name: string;
        displayName: string;
        identifier: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        displayed?: boolean;
        lifespan?: CaaSApi_Lifespan;
    }
    interface CaaSApi_SectionReference {
        fsType: "SectionReference";
        name: string;
        displayName: string;
        identifier: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        displayed?: boolean;
        lifespan?: CaaSApi_Lifespan;
    }
    interface CaaSApi_Body {
        fsType: "Body" | "GCABody";
        name: string;
        identifier: string;
        children: (CaaSApi_Section | CaaSApi_Content2Section | CaaSApi_SectionReference)[];
    }
    interface CaaSApi_GCAPage {
        _id: string;
        fsType: "GCAPage";
        name: string;
        displayName: string;
        identifier: string;
        uid: string;
        uidType: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        metaFormData: CaaSApi_DataEntries;
        children: CaaSApi_Body[];
    }
    interface CaaSApi_ProjectProperties {
        _id: string;
        fsType: "ProjectProperties";
        name: string;
        displayName: string;
        identifier: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        metaFormData: CaaSApi_DataEntries;
        projectConfiguration?: {
            masterLocale: MasterLocale;
        };
    }
    interface CaaSApi_Page {
        fsType: "Page";
        name: string;
        displayName: string;
        identifier: string;
        uid: string;
        uidType: string;
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        metaFormData: CaaSApi_DataEntries;
        children: CaaSApi_Body[];
    }
    interface CaaSApi_PageRef {
        _id: string;
        fsType: "PageRef";
        name: string;
        identifier: string;
        uid: string;
        uidType: string;
        url: string;
        page: CaaSApi_Page;
        displayName: string;
        metaFormData?: CaaSApi_DataEntries;
        route: string;
    }
    interface CaaSApi_Media_Base {
        _id: string;
        fsType: "Media";
        name: string;
        displayName: string;
        identifier: string;
        uid: string;
        uidType: string;
        fileName: string;
        languageDependent: boolean;
        description: string | null;
        metaFormData: CaaSApi_DataEntries;
        changeInfo?: {
            revision: number;
        };
    }
    interface CaaSApi_Media_File extends CaaSApi_Media_Base {
        mediaType: "FILE";
        url: string;
        fileMetaData: {
            fileSize: number;
            extension: string;
            mimeType: string;
            encoding: string | null;
        };
    }
    type CaaSApi_Media_Picture_Resolution_MetaData = {
        fileSize: number;
        extension: string;
        mimeType: string;
        width: number;
        height: number;
        url: string;
    };
    interface CaaSApiMediaPictureResolutions {
        [resolution: string]: CaaSApi_Media_Picture_Resolution_MetaData;
    }
    interface CaaSApi_Media_Picture extends CaaSApi_Media_Base {
        mediaType: "PICTURE";
        resolutionsMetaData: CaaSApiMediaPictureResolutions;
    }
    type CaaSApi_Media = CaaSApi_Media_File | CaaSApi_Media_Picture;
    interface CaasApi_FilterResponse {
        _id: string;
        _etag: {
            $oid: string;
        };
        _returned: number;
        _embedded: {
            "rh:doc": (CaaSApi_PageRef | CaaSApi_Dataset | CaaSApi_Media | CaaSApi_GCAPage | any)[];
        };
    }
    type DataEntry = any;
    interface DataEntries {
        [key: string]: DataEntry;
    }
    type PageBodyContent = Section | Content2Section | Dataset;
    interface PageBody {
        type: "PageBody";
        name: string;
        previewId: string;
        children: PageBodyContent[];
    }
    interface Page {
        type: "Page";
        id: string;
        refId: string;
        previewId: string;
        name: string;
        layout: string;
        children: PageBody[];
        data: DataEntries;
        meta: DataEntries;
        metaPageRef?: DataEntries;
        remoteProjectId?: string;
        route: string;
    }
    interface Reference {
        type: "Reference";
        referenceId: string;
        referenceType: string;
        referenceRemoteProject?: string;
        section?: CaaSApi_SectionBaseInfo;
    }
    interface Option {
        type: "Option";
        key: string;
        value: string;
    }
    interface Link {
        type: "Link";
        template: string;
        data: DataEntries;
        meta: DataEntries;
    }
    interface Card {
        type: "Card";
        id: string;
        previewId: string;
        template: string;
        data: DataEntries;
    }
    /**
     * This type is extended by ImageMapAreaCircle, ImageMapAreaRect, ImageMapAreaPoly
     */
    interface ImageMapArea {
        areaType: ImageMapAreaType;
        link: {
            template: string;
            data: DataEntries;
        } | null;
    }
    interface ImageMapAreaCircle extends ImageMapArea {
        areaType: ImageMapAreaType.CIRCLE;
        radius: number;
        center: Point2D;
    }
    interface ImageMapAreaRect extends ImageMapArea {
        areaType: ImageMapAreaType.RECT;
        leftTop: Point2D;
        rightBottom: Point2D;
    }
    interface ImageMapAreaPoly extends ImageMapArea {
        areaType: ImageMapAreaType.POLY;
        points: Point2D[];
    }
    interface ImageMap {
        type: "ImageMap";
        media: Image | null;
        areas: ImageMapArea[];
    }
    interface Media {
    }
    interface GCAPage {
        type: "GCAPage";
        id: string;
        previewId: string;
        name: string;
        layout: string;
        children: PageBody[];
        data: DataEntries;
        meta: DataEntries;
        remoteProjectId?: string;
    }
    interface ProjectProperties {
        type: "ProjectProperties";
        id: string;
        previewId: string;
        name: string;
        layout: string;
        data: DataEntries;
        meta: DataEntries;
        remoteProjectId?: string;
        masterLocale?: MasterLocale;
    }
    interface Content2Section {
        type: "Content2Section";
        sectionType: string;
        data: {
            schema: string;
            entityType: string;
            query: string | null;
            recordCountPerPage: number;
            maxPageCount: number;
            filterParams: Record<string, any>;
            ordering: {
                attribute: string;
                ascending: boolean;
            }[];
        };
        children: Dataset[];
    }
    interface Section {
        type: "Section";
        id: string;
        name?: string;
        displayName?: string;
        previewId: string;
        sectionType: string;
        data: DataEntries;
        displayed?: boolean;
        lifespan?: Lifespan;
        children: Section[];
    }
    interface Dataset {
        type: "Dataset";
        id: string;
        previewId: string;
        schema: string;
        entityType: string;
        template: string;
        children: Section[];
        data: DataEntries;
        route: string;
        routes: DatasetRoute[];
        remoteProjectId?: string;
        locale: string;
    }
    interface Image {
        type: "Image";
        id: string;
        previewId: string;
        meta: DataEntries;
        description: string | null;
        resolutions: {
            [resolution: string]: {
                fileSize: number;
                extension: string;
                mimeType: string;
                width: number;
                height: number;
                url: string;
            };
        };
        remoteProjectId?: string;
    }
    interface File {
        type: "File";
        id: string;
        previewId: string;
        meta: DataEntries;
        fileName: string;
        url: string;
        fileMetaData: {
            fileSize: number;
            extension: string;
            mimeType: string;
            encoding: string | null;
        };
        remoteProjectId?: string;
    }
    interface RegisteredDatasetQuery {
        name: string;
        filterParams: Record<string, string>;
        ordering: {
            attribute: string;
            ascending: boolean;
        }[];
        path: NestedPath;
        schema: string;
        entityType: string;
    }
    type NestedPath = (string | number)[];
    type CustomMapper = (entry: CaaSApi_DataEntry, entryPath: NestedPath, utils: {
        api: FSXARemoteApi;
        xmlParser: XMLParser;
        registerReferencedItem: (identifier: string, path: NestedPath, remoteProjectId?: string) => string;
        buildPreviewId: (identifier: string, remoteProjectLocale?: string) => string;
        buildMediaUrl: (url: string, rev?: number) => string;
        mapDataEntries: (entries: CaaSApi_DataEntries, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string) => Promise<DataEntries>;
    }) => Promise<any>;
    type MappedCaasItem = Page | GCAPage | Dataset | Image | File | ProjectProperties;
    type CaasApi_Item = CaaSApi_Dataset | CaaSApi_PageRef | CaaSApi_Media | CaaSApi_GCAPage | CaaSApi_ProjectProperties;
    interface FSXAConfiguration {
        apiKey: string;
        navigationService: string;
        caas: string;
        projectId: string;
        tenantId: string;
        contentMode?: "preview" | "release";
        customMapper?: CustomMapper;
        remotes?: RemoteProjectConfiguration;
        enableEventStream?: boolean;
    }
    interface ObjectMap<ValueType = any> {
        [key: string]: ValueType;
    }
    interface StructureItem {
        id: string;
        children: StructureItem[];
    }
    interface NavigationItem {
        id: string;
        parentIds: string[];
        label: string;
        contentReference: string | null;
        caasDocumentId: string;
        seoRoute: string;
        seoRouteRegex: string | null;
        customData: any;
        permissions?: {
            allowed: string[];
            denied: string[];
        };
    }
    interface NavigationData {
        idMap: {
            [id: string]: NavigationItem;
        };
        seoRouteMap: {
            [route: string]: string;
        };
        structure: StructureItem[];
        pages: {
            index: string;
        };
        meta: {
            identifier: {
                tenantId: string;
                navigationId: string;
                languageId: string;
            };
        };
    }
    type ComparisonFilterValue = string | number | RegExp | boolean | null;
    type ComparisonFilter = {
        field: string;
        operator: ComparisonQueryOperatorEnum.GREATER_THAN | ComparisonQueryOperatorEnum.GREATER_THAN_EQUALS | ComparisonQueryOperatorEnum.LESS_THAN | ComparisonQueryOperatorEnum.LESS_THAN_EQUALS;
        value: number | string;
    } | {
        field: string;
        operator: ComparisonQueryOperatorEnum.IN | ComparisonQueryOperatorEnum.NOT_IN;
        value: ComparisonFilterValue[];
    } | {
        field: string;
        operator: ComparisonQueryOperatorEnum.EQUALS | ComparisonQueryOperatorEnum.NOT_EQUALS;
        value: ComparisonFilterValue | ComparisonFilterValue[];
    };
    type LogicalFilter = {
        operator: LogicalQueryOperatorEnum.AND;
        filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
    } | {
        field: string;
        operator: LogicalQueryOperatorEnum.NOT;
        filter: {
            operator: ComparisonQueryOperatorEnum;
            value: any;
        };
    } | {
        operator: LogicalQueryOperatorEnum.NOR;
        filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
    } | {
        operator: LogicalQueryOperatorEnum.OR;
        filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
    };
    type ArrayFilter = {
        field: string;
        operator: ArrayQueryOperatorEnum.ALL;
        value: string[] | number[] | boolean[];
    };
    type EvaluationFilter = {
        field: string;
        operator: EvaluationQueryOperatorEnum.REGEX;
        value: string;
    };
    type QueryBuilderQuery = LogicalFilter | ComparisonFilter | ArrayFilter | EvaluationFilter;
    interface MappedFilter {
        [key: string]: MappedFilter | MappedFilter[] | ComparisonFilterValue | ComparisonFilterValue[];
    }
    interface RichTextElement {
        type: "block" | "text" | "paragraph" | "list" | "listitem" | "linebreak" | "link" | string;
        content: RichTextElement[] | string;
        data: Link | Record<string, any>;
    }
    type FetchNavigationParams = {
        locale?: string;
        initialPath?: string;
        fetchOptions?: RequestInit;
        filterContext?: unknown;
    };
    type SortParams = {
        name: string;
        order?: "asc" | "desc";
    };
    type FetchByFilterParams = {
        filters: QueryBuilderQuery[];
        locale?: string;
        page?: number;
        pagesize?: number;
        additionalParams?: Record<"keys" | string, any>;
        remoteProject?: string;
        fetchOptions?: RequestInit;
        filterContext?: unknown;
        sort?: SortParams[];
        normalized?: boolean;
    };
    type FetchElementParams = {
        id: string;
        locale: string;
        additionalParams?: Record<"keys" | string, any>;
        remoteProject?: string;
        fetchOptions?: RequestInit;
        filterContext?: unknown;
        normalized?: boolean;
    };
    type FetchProjectPropertiesParams = {
        locale: string;
        resolver?: string[];
        fetchOptions?: RequestInit;
    };
    type ConnectEventStreamParams = {
        remoteProject?: string;
    };
    interface AppContext<T = unknown> {
        app?: T;
        fsxaApi?: FSXAApi;
    }
    type RemoteProjectConfiguration = {
        [name: string]: {
            id: string;
            locale: string;
        };
    };
    type RemoteProjectConfigurationEntry = RemoteProjectConfiguration[keyof RemoteProjectConfiguration];
    interface CaasItemFilterParams<FilterContextType> extends MapResponse {
        filterContext?: FilterContextType;
        isKeysSet: boolean;
    }
    type CaasItemFilter<FilterContextType = unknown> = (params: CaasItemFilterParams<FilterContextType>) => Promise<MapResponse> | MapResponse;
    interface NavigationItemFilterParams<FilterContextType = unknown> {
        navigationItems: NavigationItem[];
        filterContext?: FilterContextType;
    }
    type NavigationItemFilter<FilterContextType = unknown> = (params: NavigationItemFilterParams<FilterContextType>) => Promise<NavigationItem[]> | NavigationItem[];
    /**
     * Options for filtering data in the {@link FSXARemoteApi FSXARemoteApi}
     *
     * @experimental
     */
    interface RemoteApiFilterOptions<FilterContextType = unknown> {
        navigationItemFilter?: NavigationItemFilter<FilterContextType>;
        caasItemFilter?: CaasItemFilter<FilterContextType>;
    }
    type FSXARemoteApiConfig = {
        apikey: string;
        caasURL: string;
        navigationServiceURL: string;
        tenantID: string;
        projectID: string;
        contentMode: FSXAContentMode;
        remotes?: RemoteProjectConfiguration;
        logLevel?: LogLevel;
        maxReferenceDepth?: number;
        customMapper?: CustomMapper;
        filterOptions?: RemoteApiFilterOptions;
        enableEventStream?: boolean;
    };
    type FilterContextProvider = () => unknown | null;
    /**
     * Options for filtering data in the {@link FSXAProxyApi FSXAProxyApi}
     *
     * @experimental
     */
    interface ProxyApiFilterOptions {
        filterContextProvider?: FilterContextProvider;
    }
    interface FSXAProxyApiConfig {
        clientUrl: string;
        serverUrl: string;
        logLevel: LogLevel;
        contentMode: FSXAContentMode;
        enableEventStream?: boolean;
    }
    type NormalizedProjectPropertyResponse = {
        projectProperties: ProjectProperties;
        projectPropertiesResolvedReferences: NormalizedFetchResponse["resolvedReferences"];
        projectPropertiesReferenceMap: NormalizedFetchResponse["referenceMap"];
        resolveItems: NormalizedFetchResponse["items"];
        resolveResolvedReferences: NormalizedFetchResponse["resolvedReferences"];
        resolveReferenceMap: NormalizedFetchResponse["referenceMap"];
        idToKeyMap: Record<string, string>;
    };
    interface FSXAApi {
        mode: "proxy" | "remote";
        fetchElement: <T = Page | GCAPage | Dataset | Image | any | null>(params: FetchElementParams) => Promise<T>;
        fetchByFilter: (params: FetchByFilterParams) => Promise<FetchResponse>;
        fetchNavigation: (params: FetchNavigationParams) => Promise<NavigationData | null>;
        fetchProjectProperties: (params: FetchProjectPropertiesParams) => Promise<ProjectProperties | NormalizedProjectPropertyResponse | null>;
        enableEventStream: (enable?: boolean) => boolean;
    }
    // Return value of RemoteAPI FetchByFilter normalized = false
    type FetchResponse = DenormalizedFetchRespone | NormalizedFetchResponse;
    interface FetchResponseBase {
        page: number;
        pagesize: number;
        totalPages?: number;
        size?: number;
    }
    interface DenormalizedFetchRespone extends FetchResponseBase {
        items: (MappedCaasItem | CaasApi_Item)[] | unknown[];
        resolvedReferences: undefined;
        referenceMap: undefined;
    }
    // Return value of RemoteAPI FetchByFilter normalized = true
    interface NormalizedFetchResponse extends FetchResponseBase {
        items: (MappedCaasItem | CaasApi_Item)[]; // Mapped Items without resolved refs --> has no circles
        resolvedReferences?: ResolvedReferencesInfo;
        referenceMap?: ReferencedItemsInfo;
    }
    const FETCH_ELEMENT_ROUTE = "/elements/:id";
    const FETCH_NAVIGATION_ROUTE = "/navigation";
    const FETCH_BY_FILTER_ROUTE = "/filter";
    const HEALTH_ROUTE = "/health";
    const STREAM_CHANGE_EVENTS_ROUTE = "/change-stream";
    const getFetchElementRoute: (id: string) => string;
    interface FetchNavigationRouteBody {
        initialPath?: string;
        locale?: string;
        filterContext?: unknown;
    }
    interface FetchElementRouteBody {
        id: string;
        locale: string;
        additionalParams?: Record<string, unknown>;
        remote?: string;
        filterContext?: unknown;
        normalized?: boolean;
    }
    interface FetchByFilterBody {
        locale: string;
        filters: QueryBuilderQuery[];
        page?: number;
        pagesize?: number;
        sort?: SortParams[];
        additionalParams: Record<string, unknown>;
        remote: string;
        filterContext?: unknown;
        normalized?: boolean;
    }
    interface FetchProjectPropertiesBody {
        locale: string;
        additionalParams: Record<string, unknown>;
        resolver?: string[];
        filterContext?: unknown;
        normalized?: boolean;
    }
}
type ParamValidationResult<T extends ParamValidationErrorReason> = ParamValidationSuccess | ParamValidationError<T>;
type ParamValidationSuccess = {
    valid: true;
};
/**
 * All Existing Reasons to reject param Validation
 */
type ParamValidationErrorReason = "MISSING_PARAM_OBJECT" | "MISSING_LOCALE" | "MISSING_ID" | "MISSING_FILTERS";
type ParamValidationError<T extends ParamValidationErrorReason = ParamValidationErrorReason> = Readonly<{
    valid: false;
    reason: T;
    statusCode: 400;
}>;
type FetchWrapperResult<T = any> = FetchWrapperSuccess<T> | FetchWrapperError;
type FetchWrapperSuccess<T = any> = {
    success: true;
    data: T;
};
type FetchWrapperError = {
    success: false;
    error: {
        statusCode: number;
        message: string;
    };
    originalError: Error | ParamValidationError;
};
type buildNavigationServiceURLParams = {
    locale?: string;
    initialPath?: string;
    all?: boolean;
};
type buildCaaSUrlParams = {
    filters?: QueryBuilderQuery[];
    locale?: string;
    page?: number;
    pagesize?: number;
    sort?: SortParams[];
    additionalParams?: Record<"keys" | string, any>;
    remoteProject?: string;
    id?: string;
};
/**
 * This class represents the `remote` variant of the FSXA API.
 */
declare class FSXARemoteApi implements FSXAApi {
    mode: "remote";
    private _apikey;
    private _caasURL;
    private _navigationServiceURL;
    private _tenantID;
    private _projectID;
    private _remotes;
    private _contentMode;
    private _maxReferenceDepth?;
    private _customMapper?;
    private _queryBuilder;
    private _logger;
    private _navigationItemFilter?;
    private _caasItemFilter?;
    private _logLevel;
    private _enableEventStream;
    /**
     * The constructor of this class initializes the configuration for the api.
     *
     * @param config {@link FSXARemoteApiConfig FSXARemoteApiConfig}
     * @param config.apikey
     * @param config.caasURL
     * @param config.navigationServiceURL
     * @param config.tenantID
     * @param config.projectID
     * @param config.remotes optional {@link RemoteProjectConfiguration RemoteProjectConfiguration}
     * @param config.contentMode 'release' | 'preview'
     * @param config.maxReferenceDepth optional number to define the maximum depth of resolved objects
     * @param config.customMapper optional {@link CustomMapper CustomMapper}
     * @param config.filterOptions optional {@link RemoteApiFilterOptions RemoteApiFilterOptions} (EXPERIMENTAL)
     * @param config.logLevel the used {@link LogLevel LogLevel} for the API `(default LogLevel.ERROR)` - optional
     */
    constructor({ apikey, caasURL, navigationServiceURL, tenantID, projectID, remotes, contentMode, maxReferenceDepth, customMapper, filterOptions, logLevel }: FSXARemoteApiConfig);
    /**
     * Can be used in CaaS requests.
     * @returns an object with the configured apikey as value for the authorization key.
     */
    get authorizationHeader(): {
        authorization: string;
    };
    private verifyRemoteProjectExists;
    private getRemoteConfigById;
    /**
     * This methods builds an URL for the CaaS.
     * Based upon the optional {@link buildCaaSUrlParams buildCaaSUrlParams} object the returning url can link to any desired document.
     * @param id a specific CaaS document id
     * @param locale the locale of CaaS document (id of the document must be set)
     * @param remoteProject name of the remote project
     * @param additionalParams additional URL parameters
     * @param filters filters for CaaS documents - for more details read the [CaaS Platform documentation](https://docs.e-spirit.com/module/caas-platform/CaaS_Platform_Documentation_EN.html#use-of-filters)
     * @param page number of the page you want to access
     * @param pagesize number of the resulting CaaS documents
     * @returns a string that contains the CaaS URL with the configured parameters as query parameters
     */
    buildCaaSUrl({ id, locale, remoteProject: remoteProjectId, additionalParams, filters, page, pagesize, sort }?: buildCaaSUrlParams): string;
    /**
     * This fuction builds the URL for the NavigationService.
     * Based upon the optional {@link buildNavigationServiceURLParams buildNavigationServiceURLParams} object the returning url can link
     * to a seo route or a locale specific subtree of the navigation.
     * For more details how the Navigation Service works,
     * read the [Navigation Service documentation](https://navigationservice.e-spirit.cloud/docs/user/en/documentation.html).
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
     * @param initialPath can be provided when you want to access a subtree of the navigation
     * @returns {string} the Navigation Service url for either a subtree of or a complete navigation
     */
    buildNavigationServiceUrl({ locale, initialPath, all }?: buildNavigationServiceURLParams): string;
    /**
     * This method fetches the navigation from the configured navigation service.
     * The {@link FetchNavigationParams FetchNavigationParams} object defines options for the navigation.
     * Check {@link buildNavigationServiceUrl buildNavigationServiceUrl} to know which URL will be used.
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
     * @param initialPath optional value can be provided when you want to access a subtree of the navigation
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @param filterContext an optional value with additional context used for filtering
     * @returns {Promise<NavigationData | null>} a Promise with the Navigation Service data or null
     */
    fetchNavigation({ locale, initialPath, fetchOptions, filterContext }: FetchNavigationParams): Promise<NavigationData | null>;
    private getFilteredNavigation;
    /**
     * This method fetches an element from the configured CaaS.
     * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
     * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
     * @typeParam T optional type parameter you can provide to get a typed CaaS object as result.
     * @param id the CaaS id of the element you want to fetch
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param additionalParams optional additional URL parameters
     * @param remoteProject optional name of the remote project
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @returns {Promise<T>} a Promise with the mapped result
     */
    fetchElement<T = MappedCaasItem | any | null>({ id, locale, additionalParams, remoteProject, fetchOptions, filterContext, normalized }: FetchElementParams): Promise<any>;
    /**
     * This method fetches a filtered page from the configured CaaS.
     * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
     * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
     * Example call:
     *
     * ```typescript
     const englishMedia = await fetchByFilter({
     filters: [
     {
     field: 'fsType',
     value: 'Media',
     operator: ComparisonQueryOperatorEnum.EQUALS,
     },
     ],
     "en_GB",
     })
     * ```
     * @param filters array of {@link QueryBuilderQuery QueryBuilderQuery} to filter you request
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param page optional the number of the page you will get results from `(default = 1)` (must be greater than 0)
     * @param pagesize optional the number of document entries you will get back `(default = 30)` (must be greater than 0)
     * @param sort optional the parameter to sort the results by `(default = [])`.
     * @param additionalParams optional additional URL parameters
     * @param remoteProject optional name of the remote project
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @returns the mapped and filtered response from the CaaS request,
     *    if `additionalParams.keys` are set, the result will be unmapped,
     *    if `data._embedded['rh:doc']` is undefined, the returning result will be the unmapped `data` object
     */
    fetchByFilter({ filters, locale, page, pagesize, additionalParams, remoteProject: remoteProjectId, fetchOptions, filterContext, sort, normalized }: FetchByFilterParams, mapper?: CaaSMapper): Promise<FetchResponse>;
    private filterMapResponse;
    // TODO: Fix unecessary array wrapping with a future major jump (as it's breaking)
    /**
     * This method fetches the project properties from the configured CaaS.
     * It uses {@link fetchByFilter fetchByFilter} to get them.
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param additionalParams optional additional URL parameters
     * @param resolve optional array of fsTypes that will be resolved `(default = 'GCAPage')`
     * @param filterContext
     * @param normalized
     * @returns the resolved project properties
     */
    fetchProjectProperties({ locale, additionalParams, resolve, filterContext, normalized }: {
        locale: string;
        additionalParams?: Record<string, any>;
        resolve?: string[];
        filterContext?: unknown;
        normalized?: boolean;
    }): Promise<ProjectProperties | NormalizedProjectPropertyResponse | null>;
    private fetchProjectPropertiesNormalized;
    /**
     * This method fetches a one-time secure token from the configured CaaS.
     * This token is used to establish the WebSocket connection.
     * @returns the secure token
     */
    fetchSecureToken(): Promise<string | null>;
    private buildStringifiedQueryParams;
    /**
     * @returns the configured apikey
     */
    get apikey(): string;
    /**
     * This method sets the string value of the apikey.
     * @param value the mandatory string value of the apikey
     */
    set apikey(value: string);
    /**
     * @returns the configured CaaS base url
     */
    get caasURL(): string;
    /**
     * This method sets the base url of the CaaS.
     * @param value the mandatory base url of the CaaS e.g. `https://customername-dev-caas-api.e-spirit.cloud`
     */
    set caasURL(value: string);
    /**
     * @returns the configured FirstSpirit project id
     */
    get projectID(): string;
    /**
     * This method sets the FirstSpirit project id
     * @param value the mandatory FirstSpirit project id
     */
    set projectID(value: string);
    /**
     * @returns the configured tenant id
     */
    get tenantID(): string;
    /**
     * This method sets the tenant id
     * @param value the mandatory tenant id (schema: `customername-stage`)
     */
    set tenantID(value: string);
    /**
     * @returns the configured NavigationService url
     */
    get navigationServiceURL(): string;
    /**
     * This method sets the url of the NavigationService.
     * @param value the mandatory url of the NavigationService e.g. `https://customername-stage-navigationservice.e-spirit.cloud/navigation`
     */
    set navigationServiceURL(value: string);
    /**
     * @returns the configured remote project configuration
     */
    get remotes(): RemoteProjectConfiguration;
    /**
     * This method sets the remote project configuration
     * @param value the mandatory remote project configuration
     * example:
     * ```typescript
     {
     "media": {
     "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
     "locale": "en_GB"
     }
     }
     ```
     */
    set remotes(value: RemoteProjectConfiguration);
    /**
     * @returns the configured content mode
     * `preview` links to the unreleased content
     * `release` links to the published content
     */
    get contentMode(): "preview" | "release";
    /**
     * This method sets the content mode
     * @param value the mandatory value of the content mode (allowed: `preview`, `release`)
     */
    set contentMode(value: "preview" | "release");
    /**
     * @returns the configured log level
     */
    get logLevel(): LogLevel;
    /**
     * Getter/Setter to enable the CaaS event stream
     * @returns true, if a event stream should pipe events from CaaS change events websocket
     */
    enableEventStream(enable?: boolean): boolean;
}
interface ReferencedItemsInfo {
    [identifier: string]: NestedPath[];
}
interface ResolvedReferencesInfo {
    [id: string]: MappedCaasItem | CaasApi_Item;
}
interface MapResponse {
    resolvedReferences: ResolvedReferencesInfo;
    mappedItems: (MappedCaasItem | CaasApi_Item)[];
    referenceMap: ReferencedItemsInfo;
}
declare class CaaSMapper {
    logger: Logger;
    api: FSXARemoteApi;
    locale: string | undefined;
    xmlParser: XMLParser;
    customMapper?: CustomMapper;
    referenceDepth: number;
    maxReferenceDepth: number;
    resolvedReferences: ResolvedReferencesInfo;
    // stores references to items of current Project
    _referencedItems: ReferencedItemsInfo;
    // stores References to remote Items
    _remoteReferences: {
        [projectId: string]: ReferencedItemsInfo;
    };
    // stores items that are being or have been processed and should not be fetched
    // Again. They cannot be added to resolvedReferences yet, since the fetch call
    // just started, when we need to keep track of them.
    _processedItems: Record<string, true>;
    constructor(api: FSXARemoteApi, locale: string | undefined, utils: {
        customMapper?: CustomMapper;
        referenceDepth?: number;
        maxReferenceDepth?: number;
    }, logger: Logger);
    addToResolvedReferences(item: MappedCaasItem | CaasApi_Item, remoteProjectId?: string): void;
    /**
     * unifies the two different id formats {id}.{locale} and {id} to {id}.{locale}
     * if remoteProjectConfiguration is passed, prefix with project id to avoid uuid clashes
     *
     * @param id uuid, may be of form {id}.{locale} or {id}
     * @param remoteProjectConfiguration remoteProjectConfig
     * @returns uuid of form {id}.{locale} or {remoteProjectId}#{id}.{locale}
     */
    unifyId(id: string, remoteProjectConfiguration?: RemoteProjectConfigurationEntry): string;
    /**
     * registers a referenced Item to be fetched later. If a remoteProjectId is passed,
     * the item will be fetched from the remote Project.
     * @param identifier item identifier
     * @param path after fetch, items are inserted at all registered paths
     * @param remoteProjectId optional. If passed, the item will be fetched from the specified project
     * @returns placeholder string
     */
    registerReferencedItem(identifier: string, path: NestedPath, remoteProjectId?: string, imageMapResolution?: string): string;
    buildPreviewId(identifier: string, remoteProjectLocale?: string): string;
    buildMediaUrl(url: string, rev?: number): string;
    mapDataEntry(entry: CaaSApi_DataEntry, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<DataEntry>;
    mapLinksInRichTextElements(richTextElements: RichTextElement[], path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<RichTextElement[]>;
    _mapPermissionGroup(group: CaaSAPI_PermissionGroup): PermissionGroup;
    mapDataEntries(entries: CaaSApi_DataEntries, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<DataEntries>;
    mapSection(section: CaaSApi_Section | CaaSApi_SectionReference, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Section>;
    mapContent2Section(content2Section: CaaSApi_Content2Section, remoteProjectLocale?: string): Promise<Section>;
    mapBodyContent(content: CaaSApi_Content2Section | CaaSApi_Section | CaaSApi_SectionReference, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<PageBodyContent>;
    mapPageBody(body: CaaSApi_Body, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<PageBody>;
    mapPageRef(pageRef: CaaSApi_PageRef, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Page>;
    mapProjectProperties(properties: CaaSApi_ProjectProperties, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ProjectProperties>;
    mapImageMapArea(area: CaaSApi_ImageMapArea, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ImageMapArea | null>;
    mapImageMap(imageMap: CaaSApi_CMSImageMap, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<ImageMap>;
    mapGCAPage(gcaPage: CaaSApi_GCAPage, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<GCAPage>;
    mapDataset(dataset: CaaSApi_Dataset, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Dataset>;
    mapMediaPicture(item: CaaSApi_Media_Picture, path?: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Image>;
    mapMediaPictureResolutionUrls(resolutions: CaaSApiMediaPictureResolutions, rev?: number): CaaSApiMediaPictureResolutions;
    mapMediaFile(item: CaaSApi_Media_File, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<File>;
    mapMedia(item: CaaSApi_Media, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string): Promise<Image | File | null>;
    // reassigned locale to avoid wrong preview ids
    // each item has its own locale
    private setLocaleFromCaasItem;
    mapFilterResponse(unmappedItems: (CaasApi_Item | any)[], additionalParams?: Record<string, any>, filterContext?: unknown, remoteProjectLocale?: string, remoteProjectId?: string): Promise<MapResponse>;
    /**
     * Calls ResolveReferences for currentProject and each RemoteProject
     *
     * @param data
     * @returns data
     */
    resolveAllReferences(filterContext?: unknown): Promise<void>;
    /**
     * This method will create a filter for all referenced items that are registered inside of the referencedItems
     * and fetch them in a single CaaS-Request. If remoteProjectId is set, referenced Items from the remoteProject are fetched
     * After a successful fetch all references in the json structure will be replaced with the fetched and mapped item
     */
    resolveReferencesPerProject(remoteProjectId?: string, filterContext?: unknown): Promise<void>;
    private getRemoteConfigForProject;
}
declare enum ComparisonQueryOperatorEnum {
    GREATER_THAN_EQUALS = "$gte",
    GREATER_THAN = "$gt",
    EQUALS = "$eq",
    IN = "$in",
    LESS_THAN = "$lt",
    LESS_THAN_EQUALS = "$lte",
    NOT_EQUALS = "$ne",
    NOT_IN = "$nin"
}
declare enum EvaluationQueryOperatorEnum {
    REGEX = "$regex"
}
declare enum LogicalQueryOperatorEnum {
    AND = "$and",
    NOT = "$not",
    NOR = "$nor",
    OR = "$or"
}
declare enum ArrayQueryOperatorEnum {
    ALL = "$all"
}
declare enum QueryBuilderErrors {
    MISSING_OPERATOR = "You have to specify an operator",
    UNKNOWN_OPERATOR = "Unknown operator passed",
    MISSING_FIELD = "The filter query requires a field",
    MISSING_VALUE = "The filter query requires a value",
    MISSING_FILTERS = "No filters property was specified",
    NOT_A_NUMBER = "This filter requires a number as value",
    NOT_AN_ARRAY = "This filter requires an array as value",
    NOT_A_STRING = "This filter requires a strinig as value",
    INVALID_REGEX = "This is not a valid regular expression"
}
declare class QueryBuilder {
    logger: Logger;
    constructor(logger: Logger);
    buildAll(filters: QueryBuilderQuery[]): MappedFilter[];
    build(filter: QueryBuilderQuery): MappedFilter | null;
}
type FSXAApiSingletonInitOptions = {
    logLevel?: LogLevel;
    enableEventStream?: boolean;
};
interface MasterLocale {
    country: string;
    language: string;
    identifier: string;
}
interface CaaSApi_Lifespan {
    start: string;
    end?: string;
}
interface Lifespan {
    start: Date;
    end?: Date;
}
interface CaaSApi_Template {
    fsType: "PageTemplate" | "SectionTemplate" | "LinkTemplate";
    name: string;
    identifier: string;
    uid: string;
    uidType: string;
    displayName: string;
}
interface CaaSApi_Option {
    fsType: "Option";
    label: string;
    identifier: string;
}
interface CaaSApi_CMSInputCheckbox {
    fsType: "CMS_INPUT_CHECKBOX";
    name: string;
    value: CaaSApi_Option[];
}
interface CaaSApi_CMSInputRadioButton {
    fsType: "CMS_INPUT_RADIOBUTTON";
    name: string;
    value: CaaSApi_Option | null;
}
interface CaaSApi_CMSInputText {
    fsType: "CMS_INPUT_TEXT";
    name: string;
    value: string;
}
interface CaaSApi_CMSInputTextArea {
    fsType: "CMS_INPUT_TEXTAREA";
    name: string;
    value: string;
}
interface CaaSApi_CMSInputDOM {
    fsType: "CMS_INPUT_DOM";
    name: string;
    /**
     * This contains xml
     */
    value: string;
}
interface CaaSApi_CMSInputDOMTable {
    fsType: "CMS_INPUT_DOMTABLE";
    name: string;
    /**
     * This contains xml
     */
    value: string;
}
interface CaaSApi_CMSInputToggle {
    fsType: "CMS_INPUT_TOGGLE";
    name: string;
    value: boolean | null;
}
interface CaaSApi_CMSInputNumber {
    fsType: "CMS_INPUT_NUMBER";
    name: string;
    value: number;
}
interface CaaSApi_CMSInputLink {
    fsType: "CMS_INPUT_LINK";
    name: string;
    value: {
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
        metaFormData: CaaSApi_DataEntries;
    };
}
interface CaaSApi_CMSInputList {
    fsType: "CMS_INPUT_LIST";
    name: string;
    value: any[];
}
interface CaaSApi_CMSInputCombobox {
    fsType: "CMS_INPUT_COMBOBOX";
    name: string;
    value: CaaSApi_Option | null;
}
interface CaaSApi_CMSInputRadioButton {
    fsType: "CMS_INPUT_RADIOBUTTON";
    name: string;
    value: CaaSApi_Option | null;
}
interface CaaSApi_CMSInputDate {
    fsType: "CMS_INPUT_DATE";
    name: string;
    value: string | null;
}
interface Point2D {
    x: number;
    y: number;
}
interface CaaSApi_ImageMapArea {
    fsType: "Area";
    areaType: ImageMapAreaType;
    link: {
        template: CaaSApi_Template;
        formData: CaaSApi_DataEntries;
    } | null;
}
interface CaaSApi_CMSImageMap {
    fsType: "CMS_INPUT_IMAGEMAP";
    name: string;
    value: {
        fsType: "MappingMedium";
        media: CaaSApi_MediaRef;
        areas: CaaSApi_ImageMapArea[];
        resolution: {
            fsType: "Resolution";
            uid: string;
            width: number;
            height: number;
        };
    };
}
interface CaaSApi_FSDataset {
    fsType: "FS_DATASET";
    name: string;
    value: {
        fsType: "DatasetReference";
        target: {
            fsType: "Dataset";
            schema: string;
            entityType: string;
            identifier: string;
        };
    } | CaaSApi_DataEntry[] | null;
}
interface CaaSApi_FSButton {
    fsType: "FS_BUTTON";
    name: string;
    value: {};
}
interface CaaSApi_Card {
    fsType: "Card";
    identifier: string;
    uuid: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
}
interface CaaSApi_FSCatalog {
    fsType: "FS_CATALOG";
    name: string;
    value: CaaSApi_Card[] | null;
}
interface CaaSApi_Record {
    fsType: "Record";
    identifier: string;
    value: Record<string, any> | null;
}
interface CaaSApi_FSIndex {
    fsType: "FS_INDEX";
    name: string;
    dapType: string;
    value: CaaSApi_Record[];
}
interface CaaSApi_SectionBaseInfo {
    name: string;
    displayName: string;
    identifier: string;
}
interface CaaSApi_BaseRef {
    fsType: string;
    name: string;
    identifier: string;
    uid: string;
    uidType: string;
    url: string;
    remoteProject?: string;
    section?: CaaSApi_SectionBaseInfo;
}
interface CaaSApi_MediaRef extends CaaSApi_BaseRef {
    fsType: "Media";
    uidType: "MEDIASTORE_LEAF";
    mediaType: "PICTURE" | "FILE";
}
interface CaaSApi_GCARef extends CaaSApi_BaseRef {
    fsType: "GCAPage";
    uidType: "GLOBALSTORE";
    url: "";
}
interface CaaSApi_PageRefRef extends CaaSApi_BaseRef {
    fsType: "PageRef";
    uidType: "SITESTORE_LEAF";
}
interface CaaSApi_FSReference {
    fsType: "FS_REFERENCE";
    name: string;
    value: CaaSApi_BaseRef | CaaSApi_PageRefRef | CaaSApi_GCARef | CaaSApi_MediaRef | null;
}
interface CaaSAPI_PermissionGroup {
    groupName: string;
    groupPath: string;
}
interface CaaSAPI_PermissionActivity {
    activity: string;
    allowed: CaaSAPI_PermissionGroup[];
    forbidden: CaaSAPI_PermissionGroup[];
}
interface CaaSApi_CMSInputPermission {
    fsType: "CMS_INPUT_PERMISSION";
    name: string;
    value: CaaSAPI_PermissionActivity[];
}
interface PermissionGroup extends CaaSAPI_PermissionGroup {
    groupId: string;
}
interface PermissionActivity extends CaaSAPI_PermissionActivity {
    allowed: PermissionGroup[];
    forbidden: PermissionGroup[];
}
interface DatasetRoute {
    pageRef: string;
    route: string;
}
type CaaSApi_DataEntry = CaaSApi_CMSInputCheckbox | CaaSApi_CMSInputCombobox | CaaSApi_CMSInputDOM | CaaSApi_CMSInputDOMTable | CaaSApi_CMSInputLink | CaaSApi_CMSInputList | CaaSApi_CMSInputNumber | CaaSApi_CMSInputText | CaaSApi_CMSInputTextArea | CaaSApi_CMSInputToggle | CaaSApi_CMSImageMap | CaaSApi_FSButton | CaaSApi_FSCatalog | CaaSApi_FSDataset | CaaSApi_FSIndex | CaaSApi_FSReference | CaaSApi_CMSInputRadioButton | CaaSApi_CMSInputDate | CaaSApi_Option | CaaSApi_CMSInputPermission;
interface CaaSApi_DataEntries {
    [key: string]: CaaSApi_DataEntry;
}
interface CaaSApi_Content2Section {
    fsType: "Content2Section";
    identifier: string;
    schema: string;
    entityType: string;
    name: string;
    displayName: string;
    template: CaaSApi_Template;
    query: string | null;
    recordCountPerPage: number;
    maxPageCount: number;
    filterParams: {
        [key: string]: any;
    };
    ordering: {
        attribute: string;
        ascending: boolean;
    }[];
}
interface CaaSApi_Dataset {
    _id: string;
    fsType: "Dataset";
    schema: string;
    entityType: string;
    displayName: string;
    identifier: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    route: string;
    routes: DatasetRoute[];
    locale: {
        language: string;
        country: string;
    };
}
interface CaaSApi_Section {
    fsType: "Section" | "GCASection";
    name: string;
    displayName: string;
    identifier: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    displayed?: boolean;
    lifespan?: CaaSApi_Lifespan;
}
interface CaaSApi_SectionReference {
    fsType: "SectionReference";
    name: string;
    displayName: string;
    identifier: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    displayed?: boolean;
    lifespan?: CaaSApi_Lifespan;
}
interface CaaSApi_Body {
    fsType: "Body" | "GCABody";
    name: string;
    identifier: string;
    children: (CaaSApi_Section | CaaSApi_Content2Section | CaaSApi_SectionReference)[];
}
interface CaaSApi_GCAPage {
    _id: string;
    fsType: "GCAPage";
    name: string;
    displayName: string;
    identifier: string;
    uid: string;
    uidType: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    metaFormData: CaaSApi_DataEntries;
    children: CaaSApi_Body[];
}
interface CaaSApi_ProjectProperties {
    _id: string;
    fsType: "ProjectProperties";
    name: string;
    displayName: string;
    identifier: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    metaFormData: CaaSApi_DataEntries;
    projectConfiguration?: {
        masterLocale: MasterLocale;
    };
}
interface CaaSApi_Page {
    fsType: "Page";
    name: string;
    displayName: string;
    identifier: string;
    uid: string;
    uidType: string;
    template: CaaSApi_Template;
    formData: CaaSApi_DataEntries;
    metaFormData: CaaSApi_DataEntries;
    children: CaaSApi_Body[];
}
interface CaaSApi_PageRef {
    _id: string;
    fsType: "PageRef";
    name: string;
    identifier: string;
    uid: string;
    uidType: string;
    url: string;
    page: CaaSApi_Page;
    displayName: string;
    metaFormData?: CaaSApi_DataEntries;
    route: string;
}
interface CaaSApi_Media_Base {
    _id: string;
    fsType: "Media";
    name: string;
    displayName: string;
    identifier: string;
    uid: string;
    uidType: string;
    fileName: string;
    languageDependent: boolean;
    description: string | null;
    metaFormData: CaaSApi_DataEntries;
    changeInfo?: {
        revision: number;
    };
}
interface CaaSApi_Media_File extends CaaSApi_Media_Base {
    mediaType: "FILE";
    url: string;
    fileMetaData: {
        fileSize: number;
        extension: string;
        mimeType: string;
        encoding: string | null;
    };
}
type CaaSApi_Media_Picture_Resolution_MetaData = {
    fileSize: number;
    extension: string;
    mimeType: string;
    width: number;
    height: number;
    url: string;
};
interface CaaSApiMediaPictureResolutions {
    [resolution: string]: CaaSApi_Media_Picture_Resolution_MetaData;
}
interface CaaSApi_Media_Picture extends CaaSApi_Media_Base {
    mediaType: "PICTURE";
    resolutionsMetaData: CaaSApiMediaPictureResolutions;
}
type CaaSApi_Media = CaaSApi_Media_File | CaaSApi_Media_Picture;
type DataEntry = any;
interface DataEntries {
    [key: string]: DataEntry;
}
type PageBodyContent = Section | Content2Section | Dataset;
interface PageBody {
    type: "PageBody";
    name: string;
    previewId: string;
    children: PageBodyContent[];
}
interface Page {
    type: "Page";
    id: string;
    refId: string;
    previewId: string;
    name: string;
    layout: string;
    children: PageBody[];
    data: DataEntries;
    meta: DataEntries;
    metaPageRef?: DataEntries;
    remoteProjectId?: string;
    route: string;
}
interface Link {
    type: "Link";
    template: string;
    data: DataEntries;
    meta: DataEntries;
}
/**
 * This type is extended by ImageMapAreaCircle, ImageMapAreaRect, ImageMapAreaPoly
 */
interface ImageMapArea {
    areaType: ImageMapAreaType;
    link: {
        template: string;
        data: DataEntries;
    } | null;
}
interface ImageMap {
    type: "ImageMap";
    media: Image | null;
    areas: ImageMapArea[];
}
interface GCAPage {
    type: "GCAPage";
    id: string;
    previewId: string;
    name: string;
    layout: string;
    children: PageBody[];
    data: DataEntries;
    meta: DataEntries;
    remoteProjectId?: string;
}
interface ProjectProperties {
    type: "ProjectProperties";
    id: string;
    previewId: string;
    name: string;
    layout: string;
    data: DataEntries;
    meta: DataEntries;
    remoteProjectId?: string;
    masterLocale?: MasterLocale;
}
interface Content2Section {
    type: "Content2Section";
    sectionType: string;
    data: {
        schema: string;
        entityType: string;
        query: string | null;
        recordCountPerPage: number;
        maxPageCount: number;
        filterParams: Record<string, any>;
        ordering: {
            attribute: string;
            ascending: boolean;
        }[];
    };
    children: Dataset[];
}
interface Section {
    type: "Section";
    id: string;
    name?: string;
    displayName?: string;
    previewId: string;
    sectionType: string;
    data: DataEntries;
    displayed?: boolean;
    lifespan?: Lifespan;
    children: Section[];
}
interface Dataset {
    type: "Dataset";
    id: string;
    previewId: string;
    schema: string;
    entityType: string;
    template: string;
    children: Section[];
    data: DataEntries;
    route: string;
    routes: DatasetRoute[];
    remoteProjectId?: string;
    locale: string;
}
interface Image {
    type: "Image";
    id: string;
    previewId: string;
    meta: DataEntries;
    description: string | null;
    resolutions: {
        [resolution: string]: {
            fileSize: number;
            extension: string;
            mimeType: string;
            width: number;
            height: number;
            url: string;
        };
    };
    remoteProjectId?: string;
}
interface File {
    type: "File";
    id: string;
    previewId: string;
    meta: DataEntries;
    fileName: string;
    url: string;
    fileMetaData: {
        fileSize: number;
        extension: string;
        mimeType: string;
        encoding: string | null;
    };
    remoteProjectId?: string;
}
type NestedPath = (string | number)[];
type CustomMapper = (entry: CaaSApi_DataEntry, entryPath: NestedPath, utils: {
    api: FSXARemoteApi;
    xmlParser: XMLParser;
    registerReferencedItem: (identifier: string, path: NestedPath, remoteProjectId?: string) => string;
    buildPreviewId: (identifier: string, remoteProjectLocale?: string) => string;
    buildMediaUrl: (url: string, rev?: number) => string;
    mapDataEntries: (entries: CaaSApi_DataEntries, path: NestedPath, remoteProjectLocale?: string, remoteProjectId?: string) => Promise<DataEntries>;
}) => Promise<any>;
type MappedCaasItem = Page | GCAPage | Dataset | Image | File | ProjectProperties;
type CaasApi_Item = CaaSApi_Dataset | CaaSApi_PageRef | CaaSApi_Media | CaaSApi_GCAPage | CaaSApi_ProjectProperties;
interface StructureItem {
    id: string;
    children: StructureItem[];
}
interface NavigationItem {
    id: string;
    parentIds: string[];
    label: string;
    contentReference: string | null;
    caasDocumentId: string;
    seoRoute: string;
    seoRouteRegex: string | null;
    customData: any;
    permissions?: {
        allowed: string[];
        denied: string[];
    };
}
interface NavigationData {
    idMap: {
        [id: string]: NavigationItem;
    };
    seoRouteMap: {
        [route: string]: string;
    };
    structure: StructureItem[];
    pages: {
        index: string;
    };
    meta: {
        identifier: {
            tenantId: string;
            navigationId: string;
            languageId: string;
        };
    };
}
type ComparisonFilterValue = string | number | RegExp | boolean | null;
type ComparisonFilter = {
    field: string;
    operator: ComparisonQueryOperatorEnum.GREATER_THAN | ComparisonQueryOperatorEnum.GREATER_THAN_EQUALS | ComparisonQueryOperatorEnum.LESS_THAN | ComparisonQueryOperatorEnum.LESS_THAN_EQUALS;
    value: number | string;
} | {
    field: string;
    operator: ComparisonQueryOperatorEnum.IN | ComparisonQueryOperatorEnum.NOT_IN;
    value: ComparisonFilterValue[];
} | {
    field: string;
    operator: ComparisonQueryOperatorEnum.EQUALS | ComparisonQueryOperatorEnum.NOT_EQUALS;
    value: ComparisonFilterValue | ComparisonFilterValue[];
};
type LogicalFilter = {
    operator: LogicalQueryOperatorEnum.AND;
    filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
} | {
    field: string;
    operator: LogicalQueryOperatorEnum.NOT;
    filter: {
        operator: ComparisonQueryOperatorEnum;
        value: any;
    };
} | {
    operator: LogicalQueryOperatorEnum.NOR;
    filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
} | {
    operator: LogicalQueryOperatorEnum.OR;
    filters: (LogicalFilter | ComparisonFilter | ArrayFilter)[];
};
type ArrayFilter = {
    field: string;
    operator: ArrayQueryOperatorEnum.ALL;
    value: string[] | number[] | boolean[];
};
type EvaluationFilter = {
    field: string;
    operator: EvaluationQueryOperatorEnum.REGEX;
    value: string;
};
type QueryBuilderQuery = LogicalFilter | ComparisonFilter | ArrayFilter | EvaluationFilter;
interface MappedFilter {
    [key: string]: MappedFilter | MappedFilter[] | ComparisonFilterValue | ComparisonFilterValue[];
}
interface RichTextElement {
    type: "block" | "text" | "paragraph" | "list" | "listitem" | "linebreak" | "link" | string;
    content: RichTextElement[] | string;
    data: Link | Record<string, any>;
}
type FetchNavigationParams = {
    locale?: string;
    initialPath?: string;
    fetchOptions?: RequestInit;
    filterContext?: unknown;
};
type SortParams = {
    name: string;
    order?: "asc" | "desc";
};
type FetchByFilterParams = {
    filters: QueryBuilderQuery[];
    locale?: string;
    page?: number;
    pagesize?: number;
    additionalParams?: Record<"keys" | string, any>;
    remoteProject?: string;
    fetchOptions?: RequestInit;
    filterContext?: unknown;
    sort?: SortParams[];
    normalized?: boolean;
};
type FetchElementParams = {
    id: string;
    locale: string;
    additionalParams?: Record<"keys" | string, any>;
    remoteProject?: string;
    fetchOptions?: RequestInit;
    filterContext?: unknown;
    normalized?: boolean;
};
type FetchProjectPropertiesParams = {
    locale: string;
    resolver?: string[];
    fetchOptions?: RequestInit;
};
type ConnectEventStreamParams = {
    remoteProject?: string;
};
type RemoteProjectConfiguration = {
    [name: string]: {
        id: string;
        locale: string;
    };
};
type RemoteProjectConfigurationEntry = RemoteProjectConfiguration[keyof RemoteProjectConfiguration];
interface CaasItemFilterParams<FilterContextType> extends MapResponse {
    filterContext?: FilterContextType;
    isKeysSet: boolean;
}
type CaasItemFilter<FilterContextType = unknown> = (params: CaasItemFilterParams<FilterContextType>) => Promise<MapResponse> | MapResponse;
interface NavigationItemFilterParams<FilterContextType = unknown> {
    navigationItems: NavigationItem[];
    filterContext?: FilterContextType;
}
type NavigationItemFilter<FilterContextType = unknown> = (params: NavigationItemFilterParams<FilterContextType>) => Promise<NavigationItem[]> | NavigationItem[];
/**
 * Options for filtering data in the {@link FSXARemoteApi FSXARemoteApi}
 *
 * @experimental
 */
interface RemoteApiFilterOptions<FilterContextType = unknown> {
    navigationItemFilter?: NavigationItemFilter<FilterContextType>;
    caasItemFilter?: CaasItemFilter<FilterContextType>;
}
type FSXARemoteApiConfig = {
    apikey: string;
    caasURL: string;
    navigationServiceURL: string;
    tenantID: string;
    projectID: string;
    contentMode: FSXAContentMode;
    remotes?: RemoteProjectConfiguration;
    logLevel?: LogLevel;
    maxReferenceDepth?: number;
    customMapper?: CustomMapper;
    filterOptions?: RemoteApiFilterOptions;
    enableEventStream?: boolean;
};
type FilterContextProvider = () => unknown | null;
/**
 * Options for filtering data in the {@link FSXAProxyApi FSXAProxyApi}
 *
 * @experimental
 */
interface ProxyApiFilterOptions {
    filterContextProvider?: FilterContextProvider;
}
type NormalizedProjectPropertyResponse = {
    projectProperties: ProjectProperties;
    projectPropertiesResolvedReferences: NormalizedFetchResponse["resolvedReferences"];
    projectPropertiesReferenceMap: NormalizedFetchResponse["referenceMap"];
    resolveItems: NormalizedFetchResponse["items"];
    resolveResolvedReferences: NormalizedFetchResponse["resolvedReferences"];
    resolveReferenceMap: NormalizedFetchResponse["referenceMap"];
    idToKeyMap: Record<string, string>;
};
interface FSXAApi {
    mode: "proxy" | "remote";
    fetchElement: <T = Page | GCAPage | Dataset | Image | any | null>(params: FetchElementParams) => Promise<T>;
    fetchByFilter: (params: FetchByFilterParams) => Promise<FetchResponse>;
    fetchNavigation: (params: FetchNavigationParams) => Promise<NavigationData | null>;
    fetchProjectProperties: (params: FetchProjectPropertiesParams) => Promise<ProjectProperties | NormalizedProjectPropertyResponse | null>;
    enableEventStream: (enable?: boolean) => boolean;
}
// Return value of RemoteAPI FetchByFilter normalized = false
type FetchResponse = DenormalizedFetchRespone | NormalizedFetchResponse;
interface FetchResponseBase {
    page: number;
    pagesize: number;
    totalPages?: number;
    size?: number;
}
interface DenormalizedFetchRespone extends FetchResponseBase {
    items: (MappedCaasItem | CaasApi_Item)[] | unknown[];
    resolvedReferences: undefined;
    referenceMap: undefined;
}
// Return value of RemoteAPI FetchByFilter normalized = true
interface NormalizedFetchResponse extends FetchResponseBase {
    items: (MappedCaasItem | CaasApi_Item)[]; // Mapped Items without resolved refs --> has no circles
    resolvedReferences?: ResolvedReferencesInfo;
    referenceMap?: ReferencedItemsInfo;
}
/**
 * This class represents the functionality of the FSXA-API as a proxy.
 * To use this class, there must be an instance of FSXARemoteApi that is accessed here.
 * By using this class, no secrets are provided to the client.
 * @see FSXARemoteApi
 */
declare class FSXAProxyApi implements FSXAApi {
    mode: "proxy";
    private _baseUrl;
    private _method;
    private _headers;
    private _logger;
    private _enableEventStream;
    private _filterContextProvider?;
    /**
     * This method requests the current state of baseUrl
     * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
     */
    get baseUrl(): string;
    /**
     * This method sets a new state of baseUrl
     * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
     */
    set baseUrl(value: string);
    /**
     * Creates a new instance with the connection to the FSXARemoteAPI
     * @param baseUrl specifies the URL to communicate with
     * @param logLevel specifies the restrictions of logs which will be displayed
     * @param filterOptions optional {@link ProxyApiFilterOptions ProxyApiFilterOptions} (EXPERIMENTAL)
     */
    constructor(baseUrl: string, logLevel?: LogLevel, filterOptions?: ProxyApiFilterOptions);
    /**
     * This methods fetches a specific element by its Id and its fetch options
     * @param id specifies the element that needs to be fetched
     * @param locale specifies the language e.g. de_DE
     * @param additionalParams sets parameters for the fetching process
     * @param remoteProject specifies the remote Project
     * @param fetchOptions specifies the options for the fetching process
     * @returns a JSON from the fetched element
     */
    fetchElement<T = Page | GCAPage | Dataset | Image | any | null>({ id, locale, additionalParams, remoteProject, fetchOptions }: FetchElementParams): Promise<T>;
    /**
     * This method fetches elements found by a specified Filter
     * @param filters specifies the filter that needs to be fetched
     * @param locale specifies the language
     * @param page specifies the page
     * @param pagesize specifies the number of pages
     * @param additionalParams specifies additional parameters for the fetching process
     * @param remoteProject specifies the remote project for the fetching process
     * @param fetchOptions specifies options in the fetching process
     * @param sort specifies the fields to sort. Default is by id descending. Multiple sortParams are possible. First parameter is prioritized over subsequent.
     * @returns a JSON from the fetched elements
     *
     * @example
     * ```
     * const response = await api.fetchByFilter({filters: [], locale: de'DE', page: 1, pagesize: 10, additionalParams: {}, remoteProject: 'my-project'})
     * ```
     */
    fetchByFilter({ filters, locale, page, pagesize, additionalParams, remoteProject, fetchOptions, sort }: FetchByFilterParams): Promise<FetchResponse>;
    /**
     * This method fetches the navigation
     * @param initialPath to identify the navigation
     * @param locale specifies the Language
     * @param fetchOptions specifies options in the fetching Process
     * @returns a JSON from the fetching process
     */
    fetchNavigation({ initialPath, locale, fetchOptions }: FetchNavigationParams): Promise<NavigationData | null>;
    /**
     * This method fetches Project Properties
     * @param locale specifies the language
     * @param resolver
     * @param fetchOptions specifies options in the fetching process
     * @returns a JSON from the fetching results
     */
    fetchProjectProperties({ locale, resolver, fetchOptions }: FetchProjectPropertiesParams): Promise<ProjectProperties | null>;
    /**
     * Getter/Setter to enable the CaaS event stream
     * @returns true, if a event stream should pipe events from CaaS change events websocket
     */
    enableEventStream(enable?: boolean): boolean;
    /**
     * This method initilises an `EventSoure` pointing to the CaaS change event stream.
     * @param additionalParams sets parameters for the fetching process
     * @param remoteProject specifies the remote Project
     * @returns an EventSource
     */
    connectEventStream({ remoteProject }?: ConnectEventStreamParams): EventSource;
    private fetch;
}
export { FSXAProxyApi, LogLevel, ComparisonQueryOperatorEnum, EvaluationQueryOperatorEnum, LogicalQueryOperatorEnum, ArrayQueryOperatorEnum, QueryBuilderErrors, QueryBuilder };
